/*

Turtle.js: Logoish turtle graphics facilities.
Version 0.1

Copyright (c) 2011 Antonio Bonifati

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.


See also:
	http://www.cs.berkeley.edu/~bh/usermanual
	http://www.dustindiaz.com/namespace-your-javascript/
	http://dev.opera.com/articles/view/html-5-canvas-the-basics/

*/

var TURTLE = (function() {
	// private vars and methods

	function deg2rad(angle) {
		// minus because in LOGO angles are measured clockwise
		return -angle * Math.PI / 180;
	}

	return {	// public interface
		ini: function(id) {	// constructor
			// store the translation vector
			var canvas = document.getElementById(id);
			this.xo = canvas.width / 2;
			this.yo = canvas.height / 2;

			// set initial turtle state
			this.pu().home();
			this.angle = 0;

			// initialize canvas
			this.context = canvas.getContext('2d');
			this.context.beginPath();
			this.context.moveTo(this.x, this.y);

			return this;
		},

		// not a LOGO command
		draw: function() {
			with (this.context) {
				stroke();
				closePath();
			}
		},

		// PENDOWN
		pd: function() {
			this.stroke = true;
			return this;
		},

		// PENUP
		pu: function() {
			this.stroke = false;
			return this;
		},

		// FORWARD dist
		fd: function(dist) {
			with (this) {
				x += dist * Math.cos(deg2rad(angle)),
				y += dist * Math.sin(deg2rad(angle));
				with (context) {
					if (stroke) {
						lineTo(x, y);
					} else {
						moveTo(x, y);
					}
				}
			}
			return this;
		},
		// BACK dist
		bk: function(dist) {
			return this.fd(-dist);
		},
		// LEFT degrees
		lt: function(angle) {
			this.angle += angle;
			return this;
		},

		// RIGHT degrees
		rt: function(angle) {
			return this.lt(-angle);
		},

		// SETPOS pos
		setpos: function(x, y) {
			this.x = x + this.xo;
			this.y = y + this.yo;
			return this;
		},

		// HOME
		home: function() {
			this.setpos(0, 0);
			return this;
		}
	};
})();
